from leapp.actors import Actor
from leapp.libraries import stdlib
from leapp.libraries.stdlib import api
from leapp.models import DefaultBootKernelInfo, Report, ActiveKernelModulesFacts
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag
from leapp import reporting
from leapp.reporting import Report, create_report
import os

class CheckDefaultBootKernel(Actor):
    """
    Check the default boot kernel, set to UEK if BTRFS is detected.

    BRTFS is supported only in the UEK kernel in OL9. BTRFS is no
    longer available or supported in the RHCK kernel.
    """

    name = "CheckDefaultBootKernel"
    consumes = (ActiveKernelModulesFacts,)
    produces = (Report, DefaultBootKernelInfo)
    tags = (ChecksPhaseTag, IPUWorkflowTag,)

    def process(self):
        kernel_file = '/run/default_kernel_leapp'

        def check_kernel_type(kernel_version):
            if 'uek' in kernel_version.lower():
                return 'kernel-uek'
            else:
                return 'kernel'

        if os.path.exists(kernel_file):
            api.current_logger().info('Default kernel file already exists: {}. Skipping saving the default kernel.'.format(kernel_file))
            try:
                with open(kernel_file, 'r') as f:
                    saved_kernel = f.readline().strip()

                kernel_type = check_kernel_type(saved_kernel)
                api.current_logger().info('Kernel type from file: {}'.format(kernel_type))

                current_kernel_type = stdlib.run(['grubby', '--default-kernel'])['stdout'].strip()

                if 'vmlinuz-upgrade.' in current_kernel_type:
                    if kernel_type == 'kernel-uek':
                        api.current_logger().info('Kernel UEK detected as default boot kernel')
                        api.produce(DefaultBootKernelInfo(kernel_type='kernel-uek'))
                        self.produce_report('UEK')
                    else:
                        api.current_logger().info('Kernel RHCK detected as default boot kernel.')
                        api.produce(DefaultBootKernelInfo(kernel_type='kernel'))
                        self.produce_report('RHCK')
                else:
                    if check_kernel_type(current_kernel_type) == 'kernel-uek':
                        api.current_logger().info('Kernel UEK detected as default boot kernel')
                        api.produce(DefaultBootKernelInfo(kernel_type='kernel-uek'))
                        self.produce_report('UEK')
                    else:
                        api.current_logger().info('Kernel RHCK detected as default boot kernel')
                        api.produce(DefaultBootKernelInfo(kernel_type='kernel'))
                        self.produce_report('RHCK')
            
            except Exception as e:
                api.current_logger().warning('Error reading the saved kernel from {}: {}'.format(kernel_file,e))
                self.produce_inhibitor()
                return
        else:
            try:
                current_default_kernel = stdlib.run(['grubby', '--default-kernel'])['stdout'].strip()
                with open(kernel_file, 'w') as f:
                    f.write(current_default_kernel + '\n')
            except (OSError, stdlib.CalledProcessError):
                api.current_logger().warning('Failed to query grubby for default kernel', exc_info=True)
                self.produce_inhibitor()
                return

            btrfs_used = 'False'
            for fact in self.consume(ActiveKernelModulesFacts):
                for active_module in fact.kernel_modules:
                    if active_module.filename == 'btrfs':
                        btrfs_used = 'True'
                        break

            if (current_default_kernel.find('uek') != -1):
                api.current_logger().info('Kernel UEK detected as default boot kernel')
                api.produce(DefaultBootKernelInfo(kernel_type='kernel-uek'))
                self.produce_report('UEK')
            else:
                if btrfs_used == 'False':
                    api.current_logger().info('Kernel RHCK detected as default boot kernel')
                    api.produce(DefaultBootKernelInfo(kernel_type='kernel'))
                    self.produce_report('RHCK')
                else:
                    api.current_logger().info('BTRFS detected, setting UEK as OL9 default boot kernel')
                    api.produce(DefaultBootKernelInfo(kernel_type='kernel-uek'))
                    self.produce_report('UEK')

    def produce_report(self, kernel_type):
        create_report([
            reporting.Title('Default Boot Kernel'),
            reporting.Summary(
                'Setting default boot kernel in OL9 to {}'.format(kernel_type)
            ),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([reporting.Groups.KERNEL])
        ])

    def produce_inhibitor(self):
        create_report([
            reporting.Title('Default Boot Kernel'),
            reporting.Summary(
                'Unable to detect default boot kernel, is grubby installed.'
            ),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([reporting.Groups.KERNEL]),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Remediation(hint='Check if grubby is installed.')
        ])



